"""
This files contains constants for the py-Nepali project.
"""

# Nepali timezone
NEPAL_TIMEZONE = "Asia/Kathmandu"

# Nepali months in english
NEPALI_MONTHS_EN = (
    "Baishakh",
    "Jestha",
    "Ashad",
    "Sharwan",
    "Bhadra",
    "Ashwin",
    "Kartik",
    "Mangsir",
    "Poush",
    "Magh",
    "Falgun",
    "Chaitra",
)

# Nepali months in Nepali (devanagari)
NEPALI_MONTHS_NE = (
    "बैशाख",
    "जेठ",
    "असार",
    "साउन",
    "भदौ",
    "असोज",
    "कात्तिक",
    "मंसिर",
    "पुस",
    "माघ",
    "फागुन",
    "चैत",
)

# Week names in english
WEEKS_EN = (
    "Sunday",
    "Monday",
    "Tuesday",
    "Wednesday",
    "Thursday",
    "Friday",
    "Saturday",
)

# Week abbreviation name in english
WEEKS_ABBR_EN = ("Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat")

# Weeks names in Nepali (devanagari)
WEEKS_NE = (
    "आइतबार",
    "सोमबार",
    "मंगलबार",
    "बुधबार",
    "बिहीबार",
    "शुक्रबार",
    "शनिबार",
)

# Week abbreviation name in Nepali (devanagari)
WEEKS_ABBR_NE = ("आइत", "सोम", "मंगल", "बुध", "बिही", "शुक्र", "शनि")
